#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Views;
using HeuristicLab.Optimizer;

namespace HeuristicLab.Problems.GPDL.Views {
  internal class CreateProblemMenuItem : HeuristicLab.MainForm.WindowsForms.MenuItem, IOptimizerUserInterfaceItemProvider {
    private const string gpdlString = @"
PROBLEM ArtificialAnt

CODE <<
public struct Point { 
  public int x; public int y; 
  public Point(int x, int y) {
    this.x = x;
	this.y = y;
  }
};

class World {

private static string[] LosAltosTrail = new string[] {
"" ###                                          "",
""   #                                  .###..  "",
""   #                                  #    #  "",
""   #                                  #    #  "",
""   ####.#####           .############..    .  "",
""            #           .                  #  "",
""            #           #                  .  "",
""            #           #                  .  "",
""            #           #                  #  "",
""            .           #                  .  "",
""            #           .                  .  "",
""            #           .                  #  "",
""            #           #                  .  "",
""            #           #            ...###.  "",
""            .       .#...            #        "",
""            .       .                .        "",
""            #       .                .        "",
""            #       #                .#...    "",
""            #       #                    #    "",
""            #       #                    .    "",
""            #       #                    .    "",
""            #       .                ...#.    "",
""            #       .                #        "",
"" ..##..#####.       #                #        "",
"" #                  #                #        "",
"" #                  #                #        "",
"" #                  #                #        "",
"" #                  #                #        "",
"" #                  #                #        "",
"" #     .#####....##..                #        "",
"" #     #                             #        "",
"" .     #                             #        "",
"" .####..                             #        "",
""                                   #.#        "",
""                                   #          "",
""                                   #          "",
""                                   #          "",
""                                   #          "",
""                                   #          "",
""                                   #          "",
""                                   #          "",
""                              ######          "",
""                              .               "",
""                              #               "",
""                              #               "",
""                              #               "",
""                              #               "",
""                              #               "",
""                              #               "",
""                            #..               "",
""                            #                 "",
""                            #                 "",
""                            #                 "",
""                            #                 "",
""                            #                 "",
""                      .######                 "",
""                      .                       "",
""                      #                       "",
""                      #                       "",
""                      #                       "",
""                      #                       "",
""                      #                       "",
""                      #                       "",
""                      #                       "",
""                      #                       "",
""                      #                       ""
};
  private char[][] world = LosAltosTrail.Select(l=>l.ToArray()).ToArray();
  public int TotalFood = LosAltosTrail.Select(l=>l.Where(c=>c=='#').Count()).Sum();
  public int Width { get { return 100; } }
  public int Height { get { return 100; } }
  
  public void Reset() {
    world = LosAltosTrail.Select(l=>l.ToArray()).ToArray();
  }
  
  public bool FoodAt(Point p) {
    if(p.x < 0 || p.y < 0) return false;
    if(world.Length <= p.y) return false;
	if(world[p.y].Length <= p.x) return false;
    return world[p.y][p.x] == '#';
  }
  public void RemoveFoodAt(Point p) {
    if(world.Length < p.y) return;
	if(world[p.y].Length < p.x) return;
    world[p.y][p.x] = ' ';
  }
}

enum Direction {West, East, North, South};

class AntState {
  public Point Position { get; set;} 
  public int MaxSteps { get; set;}
  public Direction Direction { get; set; }
  public World World { get; set; }
  public int Steps { get; set; }
  public int FoodEaten { get; set; }
 
  public AntState() {
    Steps = 0;	
	FoodEaten = 0;
  }
 
  public void TurnLeft() {
    Steps++;
    switch(Direction) {
	  case Direction.West: Direction = Direction.South; break;
	  case Direction.South: Direction = Direction.East; break;
	  case Direction.East: Direction = Direction.North; break;
	  case Direction.North: Direction = Direction.West; break;
	}
  }
  public void TurnRight() {
    Steps++;
    switch(Direction) {
	  case Direction.West: Direction = Direction.North; break;
	  case Direction.South: Direction = Direction.West; break;
	  case Direction.East: Direction = Direction.South; break;
	  case Direction.North: Direction = Direction.East; break;
	}
  }
  public void MoveForward() { 
    Position = NextPosition; 
	Steps++; 
	if(World.FoodAt(Position)) {
    	World.RemoveFoodAt(Position);
		FoodEaten++;
    }
  }
  public Point NextPosition { 
    get { 
	    int x, y;
        switch(Direction) {
    	  case Direction.West: 
		    x = (Position.x + World.Width - 1) % World.Width;
			y = Position.y;
			return new Point(x, y);
    	  case Direction.South: 
  		    x = Position.x;
			y = (Position.y + World.Height + 1) % World.Height;
			return new Point(x, y);
    	  case Direction.East:
  		    x = (Position.x + World.Width + 1) % World.Width;
			y = Position.y;
			return new Point(x, y);
    	  case Direction.North:
  		    x = Position.x;
			y = (Position.y + World.Height - 1) % World.Height;
			return new Point(x, y);
    	}
 		return new Point(-1,-1);
    } 
  }  
}

World CreateWorld() {   
  var w = new World();
  Console.WriteLine(""Total food: ""+w.TotalFood);
  return w;
}

World world;
>>
INIT <<
  world = CreateWorld();
>>

NONTERMINALS
  Ant<<World world, AntState state>>. 
  IfFoodAhead<<World world, AntState state>>.
  Prog2<<World world, AntState state>>.
  Prog3<<World world, AntState state>>.

TERMINALS
  Left<<>> CONSTRAINTS .
  Right<<>> CONSTRAINTS .
  Move<<>> CONSTRAINTS .
 
RULES
  Ant<<World world, AntState state>> = 
    IfFoodAhead<<world, state>>
    | Prog2<<world, state>>
    | Prog3<<world, state>>
    | Left<<>>                                         SEM << state.TurnLeft(); >>
    | Right<<>>                                        SEM << state.TurnRight(); >>
    | Move<<>>                                         SEM << state.MoveForward(); >>
    .
    
  IfFoodAhead<<World world, AntState state>> =     
                                                   SEM <<
                                                      if(world.FoodAt(state.NextPosition)) {
                                                   >>
                                                   
    Ant<<world, state>>                            SEM << } else { >>
    Ant<<world, state>>                            SEM << } >> 
  .

  Prog2<<World world, AntState state>> = 
    Ant<<world, state>> Ant<<world, state>>
  .
  
  Prog3<<World world, AntState state>> =
    Ant<<world, state>> Ant<<world, state>> Ant<<world, state>>
  .


MAXIMIZE                                                   /* could also use the keyword MINIMIZE here */
  <<    
    world.Reset();
    var state = new AntState() {Position = new Point(0,0), MaxSteps=400, Direction=Direction.East, World = world};
    while(state.Steps < state.MaxSteps) {
      Ant(world, state);
    }
    return (double)state.FoodEaten;
  >>  
END ArtitificialAnt.
";

    
    public override string Name {
      get { return "Create &Problem"; }
    }
    public override IEnumerable<string> Structure {
      get { return new string[] { "&Edit" }; }
    }
    public override int Position {
      get { return 3000; }
    }
    public override string ToolTipText {
      get { return "Create a genetic programming problem from a GPDL definition."; }
    }

    protected override void OnToolStripItemSet(EventArgs e) {
      ToolStripItem.Enabled = true;
    }
    protected override void OnActiveViewChanged(object sender, EventArgs e) {
      ToolStripItem.Enabled = true;
    }
    protected override void OnViewChanged(object sender, EventArgs e) {
      base.OnViewChanged(sender, e);
      ToolStripItem.Enabled = true;
    }

    public override void Execute() {
      Utils.InstallModule("Utils", new Utils.ModuleMethodDelegate(Utils.UtilsMethod));
      Utils.InstallModule("Sets", new Utils.ModuleMethodDelegate(Sets.SetsMethod));
      Utils.InstallModule("Errors", new Utils.ModuleMethodDelegate(Errors.ErrorsMethod));

      Utils.InstallModule("GPDefLex", new Utils.ModuleMethodDelegate(GPDefLex.GPDefLexMethod));
      Utils.InstallModule("GPDefSem", new Utils.ModuleMethodDelegate(GPDefSem.GPDefSemMethod));
      Utils.InstallModule("GPDefSyn", new Utils.ModuleMethodDelegate(GPDefSyn.GPDefSynMethod));

      // --- initialize modules ---
      Utils.Modules(Utils.ModuleAction.initModule);

      Errors.PushAbortMethod(new Errors.AbortMethod(Abort));
      
      using (var src = new StringReader(gpdlString)) {
        GPDefLex.src = src;
        GPDefSyn.Parse();
      }

      GPDefLex.InitLex();
      GPDefSyn.Interpret();
      MainFormManager.MainForm.ShowContent(GPDefSem.problem);
    }

    private void Abort(string abortKind, string moduleName, string methodName, string descr) {
      
      
    }
  }
}
